extern crate core;

use anyhow::Result;
use libretrogd::states::*;
use libretrogd::system::*;
use sdl2::keyboard::Scancode;
use std::path::Path;

use crate::game::Game;
use crate::states::{GamePlayState, MainMenuState};

mod components;
mod events;
mod game;
mod logic;
mod spawning;
mod states;
mod support;
mod systems;
mod tilemap;

fn main() -> Result<()> {
    let system = SystemBuilder::new()
        .window_title("Lucky Moon Rock Bridge")
        .vsync(true)
        .show_mouse(true)
        .build()?;

    let mut game = Game::new(system)?;
    let mut states = States::new();
    states.push(MainMenuState::new())?;

    let mut is_running = true;
    let mut last_ticks = game.core.system.ticks();

    while is_running && !states.is_empty() {
        game.core.system.do_events_with(|event| {
            if let sdl2::event::Event::Quit { .. } = event {
                is_running = false;
            }
        });

        last_ticks = game.update_frame_delta(last_ticks);
        states.update(&mut game)?;
        let mut audio_device = game.core.system.audio.lock();
        game.core.system.audio_queue.apply(&mut audio_device);
        drop(audio_device);
        states.render(&mut game);
        game.core.system.display()?;
    }

    Ok(())
}
